@@ -8,9 +8,10 @@ import hashlib
 import hmac
 import time
 
+from jinja2 import ChoiceLoader
 from jinja2.exceptions import TemplateNotFound
 from shake import (url_for, from36, to36, redirect_to, Rule, StorageDict,
-    to_bytestring,)
+    to_bytestring, Render)
 
 try:
     import mailshake
@@ -20,7 +21,7 @@ except ImportError:
 from .perms import REDIRECT_FIELD_NAME, is_valid
 from .utils import (bcrypt, scrypt, split_hash, generate_salt, pbkdf2,
     hash_sha256, hash_sha512, hash_bcrypt, hash_scrypt,
-    get_user_model, LazyUser, Token, auth_render)
+    get_user_model, LazyUser, Token, auth_loader)
 
 
 __all__ = (
@@ -224,11 +225,13 @@ class Auth(object):
             "`reset_expire` parameter must be a positive number"
         
         self.render = settings.pop('render', None)
+        self.auth_render = None
         self.mailer = settings.pop('mailer', None)
         self.settings = settings
-        
         if app:
             self.init_app(app)
+        if self.render:
+            self.init_render()
     
     
     def init_app(self, app):
@@ -237,10 +240,19 @@ class Auth(object):
         """
         self.app = app
         self.render = self.render or app.render
+        self.init_render()
         
         @app.before_request
         def set_user(request):
             self.get_user(request)
+
+
+    def init_render(self):
+        loader = ChoiceLoader([
+            self.render.env.loader,
+            auth_loader,
+        ])
+        self.auth_render = Render(loader=loader)
     
 
     def _get_user(self, login):
@@ -445,13 +457,6 @@ class Auth(object):
         return user
     
 
-    def render_template(self, template, context):
-        try:
-            return self.render(template, context)
-        except TemplateNotFound:
-            return auth_render(template, context)
-    
-
     def render_to_string(self, template, context):
         try:
             return self.render(template, context, to_string=True)
@@ -488,7 +493,7 @@ class Auth(object):
             kwargs['error'] = 'FAIL'
         
         template = self.settings.sign_in_template
-        return self.render_template(template, kwargs)
+        return self.auth_render(template, kwargs)
     
 
     def sign_out_view(self, request, **kwargs):
@@ -505,7 +510,7 @@ class Auth(object):
         template = self.settings.sign_out_template
         if template:
             kwargs['auth'] = self
-            return self.render_template(template, kwargs)
+            return self.auth_render(template, kwargs)
         return redirect_to(redirect_url)
     
 
@@ -551,17 +556,17 @@ class Auth(object):
                 kwargs['ok'] = True
         
         template = self.settings.reset_password_template
-        return self.render_template(template, kwargs)
+        return self.auth_render(template, kwargs)
     
 
     def email_token(self, user, token, site_name):
         assert self.mailer
         template = self.settings.email_reset_template
-        msg = self.render_to_string(template, {
+        msg = self.auth_render(template, {
             'user': user,
             'token': token,
             'site_name': site_name,
-        })
+        }, to_string=True)
         msg = to_bytestring(msg)
 
         email_msg = mailshake.EmailMessage(
@@ -621,7 +626,7 @@ class Auth(object):
                 kwargs['ok'] = True
         
         template = self.settings.change_password_template
-        return self.render_template(template, kwargs)
+        return self.auth_render(template, kwargs)
 
 
     sign_in_controller = sign_in_view
