@@ -6,8 +6,9 @@ from datetime import datetime
 
 import pytest
 import shake_auth
-from shake import Shake
+from shake import Shake, Request, local
 from solution import SQLAlchemy
+from werkzeug.test import EnvironBuilder
 
 
 app_settings = {
@@ -58,51 +59,19 @@ def test_create_user():
     assert int(cost) == auth.settings['hash_cost']
 
 
-def test_sign_in():
+def test_sign_in_out():
+    local.app = app = Shake(__file__, app_settings)
     auth = get_auth()
-    user = auth.create_user(u'admin', u'passw')
-
-    def index(request):
-        auth.sign_in(request, user)
-        return request.user.login
-    
-    def get_user(request):
-        return request.user.login
-
-    app = Shake(__file__, app_settings)
     auth.init_app(app)
-    app.add_url('/', index)
-    app.add_url('/user/', get_user)
-    c = app.test_client()
-    
-    resp = c.get('/')
-    resp2 = c.get('/user/')
-
-    assert resp.headers.get('Set-Cookie').startswith('shake_session')
-    assert resp.data == user.login
-    assert resp2.data == resp.data
-
-
-def test_sign_out():
-    auth = get_auth()
     user = auth.create_user(u'admin', u'passw')
-
-    def index(request):
-        auth.sign_in(request, user)
-        assert request.user
+    env = get_test_env()
     
-    def sign_out(request):
-        auth.sign_out(request)
-        assert request.user is None
-
-    app = Shake(__file__, app_settings)
-    auth.init_app(app)
-    app.add_url('/', index)
-    app.add_url('/sign_out/', sign_out)
-    c = app.test_client()
-
-    resp = c.get('/')
-    resp = c.get('/sign_out/')
+    request = Request(env)
+    auth.sign_in(request, user)
+    assert request.user.login
+    
+    auth.sign_out(request)
+    assert request.user is None
 
 
 def test_authenticate_by_password():
@@ -172,3 +141,14 @@ def test_authenticate_and_update():
     alg, cost, hashed = user.password.split('$')
     assert alg == 'bcrypt'
 
+
+# -----------------------------------------------------------------------------
+
+def get_test_env(path='/', **kwargs):
+    builder = EnvironBuilder(path=path, **kwargs)
+    return builder.get_environ()
+
+
+def get_test_request(path='/', **kwargs):
+    env = get_test_env(path, **kwargs)
+    return Request(env)
